@@ -188,21 +188,25 @@ void die(const char *fmt, ...)
 static unsigned int pre_buffer_size;
 static char pre_buffer[8192];
 
-int Wdefault_bitfield_sign = 0;
-int Wone_bit_signed_bitfield = 1;
-int Wcast_truncate = 1;
-int Wbitwise = 0;
-int Wtypesign = 0;
-int Wcontext = 1;
-int Wundefined_preprocessor = 0;
-int Wptr_subtraction_blows = 0;
-int Wcast_to_address_space = 0;
-int Wdecl = 0;
-int Wtransparent_union = 1;
-int Wshadow = 0;
 int Waddress_space = 1;
+int Wbitwise = 0;
+int Wcast_to_as = 0;
+int Wcast_truncate = 1;
+int Wcontext = 1;
+int Wdecl = 1;
+int Wdefault_bitfield_sign = 0;
+int Wdo_while = 0;
 int Wenum_mismatch = 1;
-int Wdo_while = 1;
+int Wnon_pointer_null = 1;
+int Wold_initializer = 1;
+int Wone_bit_signed_bitfield = 1;
+int Wparen_string = 0;
+int Wptr_subtraction_blows = 0;
+int Wreturn_void = 0;
+int Wshadow = 0;
+int Wtransparent_union = 1;
+int Wtypesign = 0;
+int Wundef = 0;
 int Wuninitialized = 1;
 
 int dbg_entry = 0;
@@ -250,7 +254,8 @@ static char **handle_switch_D(char *arg, char **next)
 
 static char **handle_switch_E(char *arg, char **next)
 {
-	preprocess_only = 1;
+	if (arg[1] == '\0')
+		preprocess_only = 1;
 	return next;
 }
 
@@ -320,6 +325,11 @@ static char **handle_switch_m(char *arg, char **next)
 		max_int_alignment = 8;
 		bits_in_pointer = 64;
 		pointer_alignment = 8;
+		size_t_ctype = &ulong_ctype;
+		ssize_t_ctype = &long_ctype;
+	} else if (!strcmp(arg, "msize-long")) {
+		size_t_ctype = &ulong_ctype;
+		ssize_t_ctype = &long_ctype;
 	}
 	return next;
 }
@@ -336,21 +346,25 @@ static const struct warning {
 	const char *name;
 	int *flag;
 } warnings[] = {
-	{ "cast-to-as", &Wcast_to_address_space },
-	{ "decl", &Wdecl },
-	{ "one-bit-signed-bitfield", &Wone_bit_signed_bitfield },
-	{ "cast-truncate", &Wcast_truncate },
-	{ "ptr-subtraction-blows", &Wptr_subtraction_blows },
-	{ "default-bitfield-sign", &Wdefault_bitfield_sign },
-	{ "undef", &Wundefined_preprocessor },
-	{ "bitwise", &Wbitwise },
-	{ "typesign", &Wtypesign },
-	{ "context", &Wcontext },
-	{ "transparent-union", &Wtransparent_union },
-	{ "shadow", &Wshadow },
 	{ "address-space", &Waddress_space },
-	{ "enum-mismatch", &Wenum_mismatch },
+	{ "bitwise", &Wbitwise },
+	{ "cast-to-as", &Wcast_to_as },
+	{ "cast-truncate", &Wcast_truncate },
+	{ "context", &Wcontext },
+	{ "decl", &Wdecl },
+	{ "default-bitfield-sign", &Wdefault_bitfield_sign },
 	{ "do-while", &Wdo_while },
+	{ "enum-mismatch", &Wenum_mismatch },
+	{ "non-pointer-null", &Wnon_pointer_null },
+	{ "old-initializer", &Wold_initializer },
+	{ "one-bit-signed-bitfield", &Wone_bit_signed_bitfield },
+	{ "paren-string", &Wparen_string },
+	{ "ptr-subtraction-blows", &Wptr_subtraction_blows },
+	{ "return-void", &Wreturn_void },
+	{ "shadow", &Wshadow },
+	{ "transparent-union", &Wtransparent_union },
+	{ "typesign", &Wtypesign },
+	{ "undef", &Wundef },
 	{ "uninitialized", &Wuninitialized },
 };
 
@@ -546,8 +560,11 @@ void declare_builtin_functions(void)
 {
 	/* Gaah. gcc knows tons of builtin <string.h> functions */
 	add_pre_buffer("extern void *__builtin_memcpy(void *, const void *, __SIZE_TYPE__);\n");
+	add_pre_buffer("extern void *__builtin_mempcpy(void *, const void *, __SIZE_TYPE__);\n");
 	add_pre_buffer("extern void *__builtin_memset(void *, int, __SIZE_TYPE__);\n");	
 	add_pre_buffer("extern int __builtin_memcmp(const void *, const void *, __SIZE_TYPE__);\n");	
+	add_pre_buffer("extern char *__builtin_strcat(char *, const char *);\n");
+	add_pre_buffer("extern char *__builtin_strncat(char *, const char *, __SIZE_TYPE__);\n");
 	add_pre_buffer("extern int __builtin_strcmp(const char *, const char *);\n");
 	add_pre_buffer("extern char *__builtin_strchr(const char *, int);\n");
 	add_pre_buffer("extern char *__builtin_strcpy(char *, const char *);\n");
@@ -555,6 +572,7 @@ void declare_builtin_functions(void)
 	add_pre_buffer("extern __SIZE_TYPE__ __builtin_strspn(const char *, const char *);\n");
 	add_pre_buffer("extern __SIZE_TYPE__ __builtin_strcspn(const char *, const char *);\n");
 	add_pre_buffer("extern char * __builtin_strpbrk(const char *, const char *);\n");
+	add_pre_buffer("extern __SIZE_TYPE__ __builtin_strlen(const char *);\n");
 
 	/* And some random ones.. */
 	add_pre_buffer("extern void *__builtin_return_address(unsigned int);\n");
@@ -573,6 +591,7 @@ void declare_builtin_functions(void)
 	add_pre_buffer("extern long __builtin_alpha_insql(long, long);\n");
 	add_pre_buffer("extern long __builtin_alpha_inslh(long, long);\n");
 	add_pre_buffer("extern long __builtin_alpha_cmpbge(long, long);\n");
+	add_pre_buffer("extern long __builtin_labs(long);\n");
 }
 
 void create_builtin_stream(void)
@@ -588,7 +607,10 @@ void create_builtin_stream(void)
 	// it is "long unsigned int".  In either case we can probably
 	// get away with this.  We need the #weak_define as cgcc will define
 	// the right __SIZE_TYPE__.
-	add_pre_buffer("#weak_define __SIZE_TYPE__ long unsigned int\n");
+	if (size_t_ctype == &ulong_ctype)
+		add_pre_buffer("#weak_define __SIZE_TYPE__ long unsigned int\n");
+	else
+		add_pre_buffer("#weak_define __SIZE_TYPE__ unsigned int\n");
 	add_pre_buffer("#weak_define __STDC__ 1\n");
 
 	add_pre_buffer("#define __builtin_stdarg_start(a,b) ((a) = (__builtin_va_list)(&(b)))\n");
@@ -598,12 +620,9 @@ void create_builtin_stream(void)
 	add_pre_buffer("#define __builtin_va_arg_incr(x) ((x) + 1)\n");
 	add_pre_buffer("#define __builtin_va_copy(dest, src) ({ dest = src; (void)0; })\n");
 	add_pre_buffer("#define __builtin_va_end(arg)\n");
-	add_pre_buffer("#define __builtin_offsetof(type, name) ((__SIZE_TYPE__)&((type *)(0ul))->name)\n");
 
 	/* FIXME! We need to do these as special magic macros at expansion time! */
 	add_pre_buffer("#define __BASE_FILE__ \"base_file.c\"\n");
-	add_pre_buffer("#define __DATE__ \"??? ?? ????\"\n");
-	add_pre_buffer("#define __TIME__ \"??:??:??\"\n");
 
 	if (optimize)
 		add_pre_buffer("#define __OPTIMIZE__ 1\n");
@@ -733,7 +752,7 @@ struct symbol_list *sparse_initialize(int argc, char **argv, struct string_list
 	return list;
 }
 
-struct symbol_list * __sparse(char *filename)
+struct symbol_list * sparse_keep_tokens(char *filename)
 {
 	struct symbol_list *res;
 
@@ -743,6 +762,17 @@ struct symbol_list * __sparse(char *filename)
 	new_file_scope();
 	res = sparse_file(filename);
 
+	/* And return it */
+	return res;
+}
+
+
+struct symbol_list * __sparse(char *filename)
+{
+	struct symbol_list *res;
+
+	res = sparse_keep_tokens(filename);
+
 	/* Drop the tokens for this file after parsing */
 	clear_token_alloc();
 
